from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.events.findings.application_security_posture_finding_type_id import (
    ApplicationSecurityPostureFindingTypeID,
)
from py_ocsf_models.events.findings.category_uid import CategoryUID
from py_ocsf_models.events.findings.class_uid import ClassUID
from py_ocsf_models.events.findings.finding import Finding
from py_ocsf_models.objects.device import Device
from py_ocsf_models.objects.evidence_artifacts import EvidenceArtifacts
from py_ocsf_models.objects.finding_info import FindingInformation
from py_ocsf_models.objects.remediation import Remediation
from py_ocsf_models.objects.resource_details import ResourceDetails
from py_ocsf_models.objects.vulnerability_details import VulnerabilityDetails


class ApplicationSecurityPostureFinding(Finding, BaseModel):
    """
    The Application Security Posture Finding event is a notification about any bug, defect, deficiency, exploit, vulnerability, weakness or any other issue with software and related systems. Application Security Posture Findings typically involve reporting on the greater context including compliance, impacted resources, remediation guidance, specific code defects, and/or vulnerability metadata. Application Security Posture Findings can be reported by Threat & Vulnerability Management (TVM) tools, Application Security Posture Management (ASPM) tools, or other similar tools. Note: if the event producer is a security control, the security_control profile should be applied and its attacks information, if present, should be duplicated into the finding_info object.

    Attributes:
        Category (category_name): The event category name, as defined by CategoryUID.Findings.value.
        Class (class_name) [Optional]: The event class name, as defined by ClassUID.ApplicationSecurityPostureFinding.value.
        Class ID (class_uid): The unique identifier of a class. A Class describes the attributes available in an event.
        Device (device) [Optional]: The device object.
        Finding Information (finding_info): Describes metadata related to a security finding generated by a security tool or system.
        Remediation Guidance (remediation) [Optional]: Suggested steps to address the finding.
        Resources (resources) [Optional]: Describes details about resources that were the target of the activity that triggered the finding.
        Time (time) [Required]: The standardized time when the event occurred or the finding was created.
        Timezone Offset (timezone_offset) [Optional]: Difference in minutes from UTC.
        Type ID (type_uid): The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: class_uid * 100 + activity_id.
        Type Name (type_name) [Optional]: The event/finding type name, as defined by the type_uid.
        Vulnerabilities (vulnerabilities) [Optional]: Vulnerabilities identified in the finding.
    """

    category_name: str = CategoryUID.Findings.name
    category_uid: int = CategoryUID.Findings.value
    class_name: Optional[str] = ClassUID.ApplicationSecurityPostureFinding.name
    class_uid: int = ClassUID.ApplicationSecurityPostureFinding
    device: Optional[Device] = None
    evidences: Optional[list[EvidenceArtifacts]] = None
    finding_info: FindingInformation
    remediation: Optional[Remediation] = None
    resources: Optional[list[ResourceDetails]] = None
    time: int
    time_dt: Optional[datetime] = None
    timezone_offset: Optional[int] = None
    type_uid: ApplicationSecurityPostureFindingTypeID
    type_name: Optional[str] = None
    vulnerabilities: Optional[list[VulnerabilityDetails]] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.events.findings.severity_id import SeverityID
from py_ocsf_models.events.findings.status_id import StatusID
from py_ocsf_models.objects.enrichment import Enrichment
from py_ocsf_models.objects.metadata import Metadata
from py_ocsf_models.objects.observable import Observable


class BaseEvent(BaseModel):
    """
    The base event is a generic and concrete event. It also defines a set of attributes available in most event classes. As a generic event that does not belong to any event category, it could be used to log events that are not otherwise defined by the schema.

    Attributes:
    - Enrichments (enrichments) [Optional]: Additional information from external sources associated with the finding.
    - Message (message) [Optional]: Description of the event/finding as defined by the source.
    - Metadata (metadata) [Required]: Data providing context for the event/finding.
    - Observables (observables) [Optional]: Observable elements associated with the event/finding.
    - Raw Data (raw_data) [Optional]: Original data as received from the source.
    - Severity (severity) [Optional]: The event/finding severity, normalized to the caption of the severity_id value. In the case of 'Other', it is defined by the source.
    - Severity ID (severity_id) [Required]: The level of severity assigned to the event/finding.
    - Status (status) [Optional]: The normalized status of the Finding set by the consumer normalized to the caption of the status_id value. In the case of 'Other', it is defined by the source.
    - Status Code (status_code) [Optional]: The event status code, as reported by the event source. For example, in a Windows Failed Authentication event, this would be the value of 'Failure Code', e.g. 0x18.
    - Status Details (status_detail) [Optional]: The status details contains additional information about the event/finding outcome.
    - Status ID (status_id) [Optional]: The normalized status identifier of the Finding, set by the consumer.
    - Unmapped Data (unmapped) [Optional]: The attributes that are not mapped to the event schema. The names and values of those attributes are specific to the event source.
    """

    enrichments: Optional[list[Enrichment]] = None
    message: Optional[str] = None
    metadata: Metadata
    observables: Optional[list[Observable]] = None
    raw_data: Optional[str] = None
    severity_id: SeverityID
    severity: Optional[str] = None
    status: Optional[str] = None
    status_code: Optional[str] = None
    status_detail: Optional[str] = None
    status_id: Optional[StatusID] = None
    unmapped: Optional[object] = None



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.events.findings.category_uid import CategoryUID
from py_ocsf_models.events.findings.class_uid import ClassUID
from py_ocsf_models.events.findings.compliance_finding_type_id import (
    ComplianceFindingTypeID,
)
from py_ocsf_models.events.findings.finding import Finding
from py_ocsf_models.objects.compliance import Compliance
from py_ocsf_models.objects.device import Device
from py_ocsf_models.objects.evidence_artifacts import EvidenceArtifacts
from py_ocsf_models.objects.finding_info import FindingInformation
from py_ocsf_models.objects.remediation import Remediation
from py_ocsf_models.objects.resource_details import ResourceDetails


class ComplianceFinding(Finding, BaseModel):
    """
    A Compliance Finding describe results of evaluations performed against resources, to check compliance with various Industry Frameworks or Security Standards such as NIST SP 800-53, CIS AWS Foundations Benchmark v1.4.0, ISO/IEC 27001 etc. Note: if the event producer is a security control, the security_control profile should be applied and its attacks information, if present, should be duplicated into the finding_info object. Note: If the Finding is an incident, i.e. requires incident workflow, also apply the incident profile or aggregate this finding into an Incident Finding.

    Attributes:
        Category (category_name): The event category name, as defined by CategoryUID.Findings.value.
        Class (class_name) [Optional]: The event class name, as defined by ClassUID.ComplianceFinding.value.
        Class ID (class_uid): The unique identifier of a class. A Class describes the attributes available in an event.
        Compliance (compliance): The compliance object.
        Device (device) [Optional]: The device object.
        Evidences (evidences) [Optional]: Artifacts related to the security detection activities.
        Finding Information (finding_info): Describes metadata related to a security finding generated by a security tool or system.
        Remediation Guidance (remediation) [Optional]: Suggested steps to address the finding.
        Resources (resources) [Optional]: Describes details about resources that were the target of the activity that triggered the finding.
        Time (time) [Required]: The standardized time when the event occurred or the finding was created.
        Timezone Offset (timezone_offset) [Optional]: Difference in minutes from UTC.
        Type ID (type_uid): The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: class_uid * 100 + activity_id.
        Type Name (type_name) [Optional]: The event/finding type name, as defined by the type_uid.
    """

    category_name: str = CategoryUID.Findings.name
    category_uid: int = CategoryUID.Findings.value
    class_name: Optional[str] = ClassUID.ComplianceFinding.name
    class_uid: int = ClassUID.ComplianceFinding
    compliance: Compliance
    device: Optional[Device] = None
    evidences: Optional[list[EvidenceArtifacts]] = None
    finding_info: FindingInformation
    remediation: Optional[Remediation] = None
    resources: Optional[list[ResourceDetails]] = None
    time: int
    time_dt: Optional[datetime] = None
    timezone_offset: Optional[int] = None
    type_uid: ComplianceFindingTypeID
    type_name: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.kb_article import KBArticle


class Remediation(BaseModel):
    """
    Describes the remediation strategy for addressing findings, including detailed descriptions, related knowledgebase (KB) articles,
    and external references. This class supports comprehensive remediation planning and documentation.

    Attributes:
    - desc (str): Detailed description of the remediation strategy.
    - kb_article_list (Optional[list[KBArticle]]): A list of KB articles describing patches or updates related to the remediation.
    - references (Optional[list[str]]): URLs or references supporting the described remediation strategy.
    """

    desc: str
    kb_article_list: Optional[list[KBArticle]] = None
    references: Optional[list[str]] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.container import Container


class RequestElements(BaseModel):
    """
    Represents the elements of an API request, especially in containerized applications.
    It includes details about the containers involved, any additional data associated with the request,
    communication flags, and a unique identifier for the request.

    Attributes:
    - containers: An array of containers involved in the API request/response process.
    - data: Additional JSON-formatted data associated with the API request.
    - flags: A list of communication flags indicating specific characteristics or behaviors of the request.
    - uid: A unique identifier for the API request.
    """

    containers: Optional[list[Container]] = None
    data: Optional[dict[str, object]] = None
    flags: Optional[list[str]] = None
    uid: str



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.objects.analytic import Analytic
from py_ocsf_models.objects.kill_chain_phase import KillChainPhase
from py_ocsf_models.objects.mitre_attack import MITREAttack
from py_ocsf_models.objects.related_event import RelatedEvent


class FindingInformation(BaseModel):
    """
    Finding Information describes metadata related to a security finding generated by a security tool or system.

    Attributes:
    - Analytic (analytic) [Recommended]: The analytic technique used to analyze and derive insights from the data or information that led to the finding or conclusion.
    - Created Time (created_time) [Optional]: The time when the finding was created in datetime format.
    - Created Time DT (created_time_dt) [Optional]: The time when the finding was created in datetime format.
    - Data Sources (data_sources) [Optional]: A list of data sources utilized in generation of the finding.
    - Description (desc) [Optional]: The description of the reported finding.
    - First Seen (first_seen_time) [Optional]: The time when the finding was first observed.
    - First Seen DT (first_seen_time_dt) [Optional]: The time when the finding was first observed in datetime format.
    - Kill Chain (kill_chain) [Optional]: The Cyber Kill Chain® provides a detailed description of each phase and its associated activities within the broader context of a cyber attack.
    - Last Seen (last_seen_time) [Optional]: The time when the finding was last observed.
    - Last Seen DT (last_seen_time_dt) [Optional]: The time when the finding was last observed in datetime format.
    - MITRE ATT&CK® Details (attacks) [Optional]: The MITRE ATT&CK® technique and associated tactics related to the finding.
    - Modified Time (modified_time) [Optional]: The time when the finding was last modified.
    - Modified Time DT (modified_time_dt) [Optional]: The time when the finding was last modified in datetime format.
    - Related Analytics (related_analytics) [Optional]: Other analytics related to this finding.
    - Related Events (related_events) [Optional]: Describes events and/or other findings related to the finding as identified by the security product.
    - Source URL (src_url) [Optional]: The URL pointing to the source of the finding.
    - Title (title) [Required]: A title or a brief phrase summarizing the reported finding.
    - Types (types) [Optional]: One or more types of the reported finding.
    - Unique ID (uid) [Required]: The unique identifier of the reported finding.
    """

    analytic: Optional[Analytic] = None
    created_time: Optional[int] = None
    created_time_dt: Optional[datetime] = None
    data_sources: Optional[List[str]] = None
    desc: Optional[str] = None
    first_seen_time: Optional[int] = None
    first_seen_time_dt: Optional[datetime] = None
    kill_chain: Optional[List[KillChainPhase]] = None
    last_seen_time: Optional[int] = None
    last_seen_time_dt: Optional[datetime] = None
    attacks: Optional[List[MITREAttack]] = None
    modified_time: Optional[int] = None
    modified_time_dt: Optional[datetime] = None
    related_analytics: Optional[List[Analytic]] = None
    related_events: Optional[List[RelatedEvent]] = None
    src_url: Optional[str] = None
    title: str
    types: Optional[List[str]] = None
    uid: str



from enum import IntEnum
from typing import Optional

from pydantic import BaseModel


class ReputationScoreID(IntEnum):
    """
    Enum representing different reputation score identifiers.

    Attributes:
    - Unknown (0): The reputation score is unknown.
    - Very Safe (1): Long history of good behavior.
    - Safe (2): Consistently good behavior.
    - Probably Safe (3): Reasonable history of good behavior.
    - Leans Safe (4): Starting to establish a history of normal behavior.
    - May not be Safe (5): No established history of normal behavior.
    - Exercise Caution (6): Starting to establish a history of suspicious or risky behavior.
    - Suspicious/Risky (7): A site with a history of suspicious or risky behavior (spam, scam, potentially unwanted software, potentially malicious).
    - Possibly Malicious (8): Strong possibility of maliciousness.
    - Probably Malicious (9): Indicators of maliciousness.
    - Malicious (10): Proven evidence of maliciousness.
    - Other (99): The reputation score is not mapped. See the rep_score attribute, which contains a data source specific value.
    """

    Unknown = 0
    Very_Safe = 1
    Safe = 2
    Probably_Safe = 3
    Leans_Safe = 4
    May_not_be_Safe = 5
    Exercise_Caution = 6
    Suspicious_or_Risky = 7
    Possibly_Malicious = 8
    Probably_Malicious = 9
    Malicious = 10
    Other = 99


class Reputation(BaseModel):
    """
    BaseModel class for representing reputation information.

    Attributes:
    - Provider (provider) [Recommended]: The provider of the reputation information.
    - Reputation Score (base_score) [Required]: The reputation score as reported by the event source.
    - Reputation Score (score) [Optional]: The reputation score, normalized to the caption of the score_id value. In the case of 'Other', it is defined by the event source.
    - Reputation Score ID (score_id) [Required]: The normalized reputation score identifier.
    """

    provider: str
    base_score: float
    score: Optional[str] = None
    score_id: ReputationScoreID


class TypeID(IntEnum):
    """
    Enum representing different types of observable value identifiers.

    Attributes:
    - Unknown (0): Unknown observable data type.
    - Hostname (1): Unique name assigned to a device connected to a computer network. A domain name in general is an Internet address that can be resolved through the Domain Name System (DNS).
    - IP_Address (2): Internet Protocol address (IP address), in either IPv4 or IPv6 format.
    - MAC_Address (3): Media Access Control (MAC) address.
    - User_Name (4): User name.
    - Email_Address (5): Email address.
    - URL_String (6): Uniform Resource Locator (URL) string.
    - File_Name (7): File name.
    - Hash (8): Hash. A unique value that corresponds to the content of the file, image, ja3_hash or hassh found in the schema.
    - Process_Name (9): Process name.
    - Resource_UID (10): Resource unique identifier. For example, S3 Bucket name or EC2 Instance ID.
    - Endpoint (20): The Endpoint object describes a physical or virtual device that connects to and exchanges information with a computer network. Some examples of endpoints are mobile devices, desktop computers, virtual machines, embedded devices, and servers. Internet-of-Things devices—like cameras, lighting, refrigerators, security systems, smart speakers, and thermostats—are also endpoints.
    - User (21): The User object describes the characteristics of a user/person or a security principal. Defined by D3FEND d3f:UserAccount.
    - Email (22): The Email object describes the email metadata such as sender, recipients, and direction. Defined by D3FEND d3f:Email.
    - Uniform_Resource_Locator (23): The Uniform Resource Locator (URL) object describes the characteristics of a URL. Defined in RFC 1738 and by D3FEND d3f:URL.
    - File (24): The File object represents the metadata associated with a file stored in a computer system. It encompasses information about the file itself, including its attributes, properties, and organizational details. Defined by D3FEND d3f:File.
    - Process (25): The Process object describes a running instance of a launched program. Defined by D3FEND d3f:Process.
    - Geo_Location (26): The Geo Location object describes a geographical location, usually associated with an IP address. Defined by D3FEND d3f:PhysicalLocation.
    - Container (27): The Container object describes an instance of a specific container. A container is a prepackaged, portable system image that runs isolated on an existing system using a container runtime like containerd.
    - Registry_Key (28): The registry key object describes a Windows registry key. Defined by D3FEND d3f:WindowsRegistryKey.
    - Registry_Value (29): The registry value object describes a Windows registry value.
    - Fingerprint (30): The Fingerprint object provides detailed information about a digital fingerprint, which is a compact representation of data used to identify a longer piece of information, such as a public key or file content. It contains the algorithm and value of the fingerprint, enabling efficient and reliable identification of the associated data.
    - Other (99): The observable data type is not mapped. See the type attribute, which may contain data source specific value.
    """

    Unknown = 0
    Hostname = 1
    IP_Address = 2
    MAC_Address = 3
    User_Name = 4
    Email_Address = 5
    URL_String = 6
    File_Name = 7
    Hash = 8
    Process_Name = 9
    Resource_UID = 10
    Endpoint = 20
    User = 21
    Email = 22
    Uniform_Resource_Locator = 23
    File = 24
    Process = 25
    Geo_Location = 26
    Container = 27
    Registry_Key = 28
    Registry_Value = 29
    Fingerprint = 30
    Other = 99


class Observable(BaseModel):
    """
    The observable object is a pivot element that contains related information found in many places in the event.

    Attributes:
    - Name (name): The full name of the observable attribute. The name is a pointer/reference to an attribute within the event data. For example: file.name.
    - Reputation Scores (reputation) [Optional]: Contains the original and normalized reputation scores.
    - Type (type) [Optional]: The observable value type name.
    - Type ID (type_id): The observable value type identifier.
    - Value (value) [Optional]: The value associated with the observable attribute. The meaning of the value depends on the observable type. If the name refers to a scalar attribute, then the value is the value of the attribute. If the name refers to an object attribute, then the value is not populated.
    """

    name: str
    reputation: Optional[Reputation] = None
    type: Optional[str] = None
    type_id: TypeID
    value: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.api import API
from py_ocsf_models.objects.device import Device
from py_ocsf_models.objects.dns_query import DNSQuery
from py_ocsf_models.objects.resource_details import ResourceDetails
from py_ocsf_models.objects.url import URL
from py_ocsf_models.objects.verdict import VerdictID


class EvidenceArtifacts(BaseModel):
    """
    A collection of evidence artifacts associated to the activity/activities that triggered a security detection.


    Attributes:
    - API Details (api) [Recommended]: Describes details about the API call associated with the activity that triggered the detection.
    - DNS Query (query) [Recommended]: Describes details about the DNS query associated with the activity that triggered the detection.
    - Data (data) [Optional]: Additional evidence data that is not accounted for in the specific evidence attributes. Use only when absolutely necessary.
    - Device (device) [Optional]: An addressable device, computer system or host associated to the activity that triggered the detection.
    - Name (name) [Optional]: The naming convention or type identifier of the evidence associated with the security detection. For example, the @odata.type from Microsoft Graph Alerts V2 or display_name from CrowdStrike Falcon Incident Behaviors.
    - Resources (resources) [Optional]: Describes details about the cloud resources directly related to activity that triggered the detection. For resources impacted by the detection, use Affected Resources at the top-level of the finding.
    - URL (url) [Optional]: The URL that pertains to the event or object associated to the activity that triggered the detection.
    - Verdict ID (verdict_id) [Optional]: The normalized verdict (or status) ID of the evidence associated with the security detection. For example, Microsoft Graph Security Alerts contain a verdict enumeration for each type of evidence associated with the Alert. This is typically set by an automated investigation process or an analyst/investigator assigned to the finding.
    """

    api: Optional[API] = None
    # TODO
    # actor: Optional[Actor]
    # connection_info: Optional[NetworkConnectionInformation]
    data: Optional[dict[str, object]] = None
    # TODO
    # dst_endpoint: Optional[NetworkEndpoint]
    # file: Optional[File]
    # process: Optional[Process]
    # src_endpoint: Optional[NetworkEndpoint]
    device: Optional[Device] = None
    name: Optional[str] = None
    query: Optional[DNSQuery] = None
    resources: Optional[list[ResourceDetails]] = None
    url: Optional[URL] = None
    verdict_id: Optional[VerdictID] = VerdictID.Unknown



from typing import Optional

from pydantic import BaseModel


class Feature(BaseModel):
    """
    The Feature object provides information about the software product feature that generated a specific event. It encompasses details related to the capabilities, components, user interface (UI) design, and performance upgrades associated with the feature.

    Attributes:
    - Name (name) [Optional]: The name of the feature.
    - Unique ID (uid) [Optional]: The unique identifier for the feature.
    - Version (version) [Optional]: The version of the feature.
    """

    name: Optional[str] = None
    uid: Optional[str] = None
    version: Optional[str] = None


class Product(BaseModel):
    """
    The Product object describes characteristics of a software product.

    Attributes:
    - Feature (name) [Optional]: The feature that reported the event.
    - Language (lang) [Optional]: The two letter lower case language codes, as defined by ISO 639-1. For example: en (English), de (German), or fr (French).
    - Name (name) [Optional]: The name of the product.
    - Path (path) [Optional]: The installation path of the product.
    - The product CPE identifier (cpe_name) [Optional]: The Common Platform Enumeration (CPE) name as described by (NIST) For example: cpe:/a:apple:safari:16.2.
    - URL String (url_string) [Optional]: The URL pointing towards the product.
    - Unique ID (uid) [Optional]: The unique identifier of the product.
    - Vendor Name (vendor_name) [Optional]: The name of the vendor of the product.
    - Version (version) [Optional]: The version of the product, as defined by the event source. For example: 2013.1.3-beta.
    """

    feature: Optional[Feature] = None
    lang: Optional[str] = None
    name: Optional[str] = None
    path: Optional[str] = None
    cpe_name: Optional[str] = None
    url_string: Optional[str] = None
    uid: Optional[str] = None
    vendor_name: str
    version: Optional[str] = None



from enum import IntEnum


class ApplicationSecurityPostureFindingTypeID(IntEnum):
    """
    The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: class_uid * 100 + activity_id.

    """

    Unknown = 200700
    Create = 200701
    Update = 200702
    Close = 200703
    Other = 200799



from enum import IntEnum
from typing import Optional

from pydantic import BaseModel


class NetworkType(IntEnum):
    """
    Enum representing different types of network interfaces.

    Attributes:
    - Unknown (0): The type is unknown.
    - Wired (1): A wired network interface.
    - Wireless (2): A wireless network interface.
    - Mobile (3): A mobile network interface.
    - Tunnel (4): A tunnel network interface.
    - Other (99): The type is not mapped. See the type attribute, which contains a data source specific value.
    """

    Unknown = 0
    Wired = 1
    Wireless = 2
    Mobile = 3
    Tunnel = 4
    Other = 99


class NetworkInterface(BaseModel):
    """
    The Network Interface object describes the type and associated attributes of a network interface.

    Attributes:
    - Hostname (hostname) [Recommended]: The hostname associated with the network interface.
    - IP Address (ip) [Recommended]: The IP address associated with the network interface.
    - MAC Address (mac) [Recommended]: The MAC address of the network interface.
    - Name (name) [Recommended]: The name of the network interface.
    - Namespace (namespace) [Optional]: The namespace useful in merger or acquisition situations.
    - Subnet Prefix Length (subnet_prefix) [Optional]: The subnet prefix length determines the number of bits used to represent the network part of the IP address.
    - Type (type) [Optional]: The type of network interface.
    - Type ID (type_id) [Required]: The network interface type identifier.
    - Unique ID (uid) [Optional]: The unique identifier for the network interface.
    """

    hostname: str
    ip: str
    mac: str
    name: str
    namespace: Optional[str] = None
    subnet_prefix: Optional[int] = None
    type: Optional[str] = None
    type_id: NetworkType
    uid: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.account import Account
from py_ocsf_models.objects.group import Group
from py_ocsf_models.objects.ldap_person import LDAPPerson
from py_ocsf_models.objects.organization import Organization


class User(BaseModel):
    """
    The User object describes the characteristics of a user/person or a security principal. Defined by D3FEND d3f:UserAccount.

    Attributes:
    - Account (account) [Optional]: The user's account or the account associated with the user.
    - Alternate ID (uid_alt) [Optional]: The alternate user identifier. For example, the Active Directory user GUID or AWS user Principal ID.
    - Domain (domain) [Optional]: The domain where the user is defined. For example: the LDAP or Active Directory domain.
    - Email Address (email_addr) [Optional]: The user's primary email address.
    - Full Name (full_name) [Optional]: The full name of the person, as per the LDAP Common Name attribute (cn).
    - Groups (groups) [Optional]: The administrative groups to which the user belongs.
    - LDAP Person (ldap_person) [Optional]: The additional LDAP attributes that describe a person.
    - Name (name) [Recommended]: The username. For example, janedoe1.
    - Organization (org) [Optional]: Organization and org unit related to the user.
    - Type (type) [Optional]: The type of the user. For example, System, AWS IAM User, etc.
    - Type ID (type_id) [Recommended]: The account type identifier.
    - Unique ID (uid) [Recommended]: The unique user identifier. For example, the Windows user SID, ActiveDirectory DN or AWS user ARN.
    - User Credential ID (credential_uid) [Optional]: The unique identifier of the user's credential. For example, AWS Access Key ID.
    """

    account: Account
    uid_alt: Optional[str] = None
    domain: Optional[str] = None
    email_addr: Optional[str] = None
    groups: Optional[list[Group]] = None
    ldap_person: Optional[LDAPPerson] = None
    name: str
    org: Optional[Organization] = None
    type: Optional[str] = None
    type_id: int
    uid: str
    credential_uid: Optional[str] = None



from enum import IntEnum


class DispositionID(IntEnum):
    """
    Describes the outcome or action taken by a security control, such as access control checks, malware detections or various types of policy violations.

    0	Unknown The disposition is unknown.
    1	Allowed Granted access or allowed the action to the protected resource.
    2	Blocked Denied access or blocked the action to the protected resource.
    3	Quarantined A suspicious file or other content was moved to a benign location.
    4	Isolated A session was isolated on the network or within a browser.
    5	Deleted A file or other content was deleted.
    6	Dropped The request was detected as a threat and resulted in the connection being dropped.
    7	Custom Action A custom action was executed such as running of a command script. Use the message attribute of the base class for details.
    8	Approved A request or submission was approved. For example, when a form was properly filled out and submitted. This is distinct from 1 'Allowed'.
    9	Restored A quarantined file or other content was restored to its original location.
    10	Exonerated A suspicious or risky entity was deemed to no longer be suspicious (re-scored).
    11	Corrected A corrupt file or configuration was corrected.
    12	Partially Corrected A corrupt file or configuration was partially corrected.
    13	Uncorrected A corrupt file or configuration was not corrected.
    14	Delayed An operation was delayed, for example if a restart was required to finish the operation.
    15	Detected Suspicious activity or a policy violation was detected without further action.
    16	No Action The outcome of an operation had no action taken.
    17	Logged The operation or action was logged without further action.
    18	Tagged A file or other entity was marked with extended attributes.
    19	Alert The request or activity was detected as a threat and resulted in a notification but request was not blocked.
    20	Count Counted the request or activity but did not determine whether to allow it or block it.
    21	Reset The request was detected as a threat and resulted in the connection being reset.
    22	Captcha Required the end user to solve a CAPTCHA puzzle to prove that a human being is sending the request.
    23	Challenge Ran a silent challenge that required the client session to verify that it's a browser, and not a bot.
    24	Access Revoked The requestor's access has been revoked due to security policy enforcements. Note: use the Host profile if the User or Actor requestor is not present in the event class.
    25	Rejected A request or submission was rejected. For example, when a form was improperly filled out and submitted. This is distinct from 2 'Blocked'.
    26	Unauthorized An attempt to access a resource was denied due to an authorization check that failed. This is a more specific disposition than 2 'Blocked' and can be complemented with the authorizations attribute for more detail.
    27	Error An error occurred during the processing of the activity or request. Use the message attribute of the base class for details.
    99	Other The disposition is not mapped. See the disposition attribute, which contains a data source specific value.
    """

    Unknown = 0
    Allowed = 1
    Blocked = 2
    Quarantined = 3
    Isolated = 4
    Deleted = 5
    Dropped = 6
    CustomAction = 7
    Approved = 8
    Restored = 9
    Exonerated = 10
    Corrected = 11
    PartiallyCorrected = 12
    Uncorrected = 13
    Delayed = 14
    Detected = 15
    NoAction = 16
    Logged = 17
    Tagged = 18
    Alert = 19
    Count = 20
    Reset = 21
    CaptchaRequired = 22
    ChallengeRan = 23
    AccessRevoked = 24
    Rejected = 25
    Unauthorized = 26
    Error = 27
    Other = 99



from enum import IntEnum


class CategoryUID(IntEnum):
    """
    The category unique identifier of the event.

    2 Findings: Findings events report findings, detections, and possible resolutions of malware, anomalies, or other actions performed by security products.
    """

    Findings = 2



from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.objects.mitre_attack import MITREAttack
from py_ocsf_models.objects.observable import Observable


class RelatedEvent(BaseModel):
    """
    Model representing a related event.

    The RelatedEvent class encapsulates information about an event related to a security incident,
    including details such as the Cyber Kill Chain phase, MITRE ATT&CK® details, observables,
    product identifier, event type, event type identifier, and unique identifier.

    Attributes:
    - Kill Chain (kill_chain) [Optional]: The Cyber Kill Chain® provides a detailed description of each phase and its associated activities within the broader context of a cyber attack.
    - MITRE ATT&CK® Details (attacks) [Optional]: An array of MITRE ATT&CK® objects describing the tactics, techniques & sub-techniques identified by a security control or finding.
    - Observables (observables) [Optional]: The observables associated with the event or a finding.
    - Type (type) [Optional]: The type of the related event. For example: Process Activity: Launch.
    - Type ID (type_uid) [Recommended]: The unique identifier of the related event type. For example: 100701.
    - Unique ID (uid) [Required]: The unique identifier of the related event.
    """

    # TODO
    # kill_chain: Optional[List[KillChainPhase]]
    attacks: Optional[List[MITREAttack]] = None
    observables: Optional[List[Observable]] = None
    type: Optional[str] = None
    type_uid: Optional[int] = None
    uid: str



from typing import Optional
from uuid import UUID

from pydantic import BaseModel

from py_ocsf_models.objects.fingerprint import FingerPrint
from py_ocsf_models.objects.image import Image


class Container(BaseModel):
    """
    Represents a container instance within a containerized application environment, detailing its image source,
    operational parameters, and unique identifiers.

    Attributes:
    - hash: The commit or SHA256 hash of the container image.
    - image: The container image details including name and potentially its tag.
    - tag: The image tag, specifying version, format, or OS.
    - name: The name of the container.
    - network_driver: The network driver used by the container.
    - orchestrator: The orchestrator managing the container.
    - pod_uuid: The unique identifier of the pod hosting the container.
    - runtime: The container runtime backend.
    - size: The size of the container image in bytes.
    - uid: The unique identifier for this container instance.
    """

    hash: Optional[FingerPrint] = None
    image: Optional[Image] = None
    name: str
    network_driver: Optional[str] = None
    orchestrator: Optional[str] = None
    pod_uuid: Optional[UUID] = None
    runtime: Optional[str] = None
    size: Optional[int] = None
    uid: str



from enum import IntEnum


class ConfidenceID(IntEnum):
    """
    The normalized confidence refers to the accuracy of the rule that created the finding. A rule with a low confidence means that the finding scope is wide and may create finding reports that may not be malicious in nature.

    0 Unknown: The normalized confidence is unknown.
    1 Low
    2 Medium
    3 High
    99 Other: The confidence is not mapped to the defined enum values. See the confidence attribute, which contains a data source specific value.

    """

    Unknown = 0
    Low = 1
    Medium = 2
    High = 3
    Other = 99



from datetime import datetime
from enum import IntEnum
from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.objects.device_hardware_info import DeviceHardwareInfo
from py_ocsf_models.objects.geolocation import GeoLocation
from py_ocsf_models.objects.group import Group
from py_ocsf_models.objects.image import Image
from py_ocsf_models.objects.network_interface import NetworkInterface
from py_ocsf_models.objects.operating_system import OperatingSystem
from py_ocsf_models.objects.organization import Organization


class DeviceType(IntEnum):
    """
    Enum representing different types of devices.

    Attributes:
    - Unknown (0): The type is unknown.
    - Server (1): A server.
    - Desktop (2): A desktop computer.
    - Laptop (3): A laptop computer.
    - Tablet (4): A tablet computer.
    - Mobile (5): A mobile phone.
    - Virtual (6): A virtual machine.
    - IOT (7): An Internet of Things (IOT) device.
    - Browser (8): A web browser.
    - Firewall (9): A networking firewall.
    - Switch (10): A networking switch.
    - Hub (11): A networking hub.
    - Other (99): The type is not mapped. See the type attribute, which contains a data source specific value.
    """

    Unknown = 0
    Server = 1
    Desktop = 2
    Laptop = 3
    Tablet = 4
    Mobile = 5
    Virtual = 6
    IOT = 7
    Browser = 8
    Firewall = 9
    Switch = 10
    Hub = 11
    Other = 99


class RiskLevelID(IntEnum):
    """
    Enum representing different risk level IDs.

    Attributes:
    - Info (0): Informational risk level.
    - Low (1): Low risk level.
    - Medium (2): Medium risk level.
    - High (3): High risk level.
    - Critical (4): Critical risk level.
    """

    Info = 0
    Low = 1
    Medium = 2
    High = 3
    Critical = 4


class Device(BaseModel):
    """
    Represents a device, providing details such as hostname, IP address, operating system, etc.

    Attributes:
    - Alternate ID (uid_alt) [Optional]: An alternate unique identifier of the device, if any.
    - Autoscale UID (autoscale_uid) [Optional]: The unique identifier of the cloud autoscale configuration.
    - Compliant Device (is_compliant) [Optional]: Indicates if the event occurred on a compliant device.
    - Created Time (created_time) [Optional]: The time when the device was known to have been created.
    - Description (desc) [Optional]: The description of the device, as reported by the operating system.
    - Domain (domain) [Optional]: The network domain where the device resides.
    - First Seen (first_seen_time) [Optional]: The initial discovery time of the device.
    - Geo Location (location) [Optional]: The geographical location of the device.
    - Groups (groups) [Optional]: The group names to which the device belongs.
    - Hardware Info (hw_info) [Optional]: The endpoint hardware information.
    - Hostname (hostname) [Recommended]: The device hostname.
    - Hypervisor (hypervisor) [Optional]: The name of the hypervisor running on the device.
    - IMEI (imei) [Optional]: The International Mobile Station Equipment Identifier associated with the device.
    - IP Address (ip) [Recommended]: The device IP address, in either IPv4 or IPv6 format.
    - Image (image) [Optional]: The image used as a template to run the virtual machine.
    - Instance ID (instance_uid) [Recommended]: The unique identifier of a VM instance.
    - Last Seen (last_seen_time) [Optional]: The most recent discovery time of the device.
    - MAC Address (mac) [Optional]: The MAC address of the endpoint.
    - Managed Device (is_managed) [Optional]: Indicates if the event occurred on a managed device.
    - Modified Time (modified_time) [Optional]: The time when the device was last known to have been modified.
    - Name (name) [Recommended]: The alternate device name, ordinarily as assigned by an administrator.
    - Network Interface ID (interface_uid) [Recommended]: The unique identifier of the network interface.
    - Network Interface Name (interface_name) [Recommended]: The name of the network interface.
    - Network Interfaces (network_interfaces) [Optional]: The network interfaces associated with the device.
    - Network Zone (zone) [Optional]: The network zone or LAN segment.
    - OS (os) [Optional]: The endpoint operating system.
    - Organization (org) [Optional]: Organization and org unit related to the device.
    - Personal Device (is_personal) [Optional]: Indicates if the event occurred on a personal device.
    - Region (region) [Recommended]: The region where the virtual machine is located.
    - Risk Level (risk_level) [Optional]: The risk level of the device.
    - Risk Level ID (risk_level_id) [Optional]: The normalized risk level ID.
    - Risk Score (risk_score) [Optional]: The risk score as reported by the event source.
    - Subnet (subnet) [Optional]: The subnet mask.
    - Subnet UID (subnet_uid) [Optional]: The unique identifier of a virtual subnet.
    - Trusted Device (is_trusted) [Optional]: Indicates if the event occurred on a trusted device.
    - Type (type) [Optional]: The device type.
    - Type ID (type_id) [Required]: The device type ID.
    - Unique ID (uid) [Recommended]: The unique identifier of the device.
    - VLAN (vlan_uid) [Optional]: The Virtual LAN identifier.
    - VPC UID (vpc_uid) [Optional]: The unique identifier of the Virtual Private Cloud (VPC).
    """

    uid_alt: Optional[str] = None
    autoscale_uid: Optional[str] = None
    is_compliant: Optional[bool] = None
    created_time: Optional[datetime] = None
    desc: Optional[str] = None
    domain: Optional[str] = None
    first_seen_time: Optional[datetime] = None
    location: Optional[GeoLocation] = None
    groups: Optional[List[Group]] = None
    hw_info: Optional[DeviceHardwareInfo] = None
    hostname: str
    hypervisor: Optional[str] = None
    imei: Optional[str] = None
    ip: str
    image: Optional[Image] = None
    instance_uid: Optional[str] = None
    last_seen_time: Optional[datetime] = None
    mac: Optional[str] = None
    is_managed: Optional[bool] = None
    modified_time: Optional[datetime] = None
    name: str
    interface_uid: Optional[str] = None
    interface_name: Optional[str] = None
    network_interfaces: Optional[List[NetworkInterface]] = None
    zone: Optional[str] = None
    os: Optional[OperatingSystem] = None
    org: Optional[Organization] = None
    is_personal: Optional[bool] = None
    region: str
    risk_level: Optional[str] = None
    risk_level_id: Optional[RiskLevelID] = None
    risk_score: Optional[int] = None
    subnet: Optional[str] = None
    subnet_uid: Optional[str] = None
    is_trusted: Optional[bool] = None
    type: Optional[str] = None
    type_id: DeviceType
    uid: str
    vlan_uid: Optional[str] = None
    vpc_uid: Optional[str] = None



from enum import IntEnum
from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.fingerprint import FingerPrint
from py_ocsf_models.objects.remediation import Remediation


class SoftwarePackageTypeID(IntEnum):
    Unknown = 0
    Application = 1
    Operating_System = 2
    Other = 99


class AffectedSoftwarePackage(BaseModel):
    """
    The Affected Package object describes details about a software package identified as affected by a vulnerability/vulnerabilities.

    Attributes:
    - Architecture (architecture) [Recommended]: Architecture is a shorthand name describing the type of computer hardware the packaged software is meant to run on.
    - The product CPE identifier (cpe_name) [Optional]: The Common Platform Enumeration (CPE) name as described by (NIST) For example: cpe:/a:apple:safari:16.2.
    - Epoch (epoch) [Optional]: Integer The software package epoch. Epoch is a way to define weighted dependencies based on version numbers.
    - Fixed In Version (fixed_in_version) [Optional]: The software package version in which a reported vulnerability was patched/fixed.
    - Hash (hash) [Optional]: Fingerprint Cryptographic hash to identify the binary instance of a software component. This can include any component such file, package, or library.
    - Software License (license) [Optional]: The software license applied to this package.
    - Software License URL (license_url) [Optional]: The URL pointing to the license applied on package or software. This is typically a LICENSE.md file within a repository.
    - Name (name) [Required]: The software package name.
    - Package Manager (package_manager) [Optional]: The software packager manager utilized to manage a package on a system, e.g. npm, yum, dpkg etc.
    - Package Manager URL (package_manager_url) [Optional]: The URL of the package or library at the package manager, or the specific URL or URI of an internal package manager link such as AWS CodeArtifact or Artifactory.
    - Path (path) [Optional]: The installation path of the affected package.
    - Package URL (purl) [Optional]: A purl is a URL string used to identify and locate a software package in a mostly universal and uniform way across programming languages, package managers, packaging conventions, tools, APIs and databases.
    - Software Release Details (release) [Optional]: Release is the number of times a version of the software has been packaged.
    - Remediation Guidance (remediation) [Optional]: Remediation Describes the recommended remediation steps to address identified issue(s).
    - Source URL (src_url) [Optional]: The link to the specific library or package such as within GitHub, this is different from the link to the package manager where the library or package is hosted.
    - Type (type) [Optional]: The type of software package, normalized to the caption of the type_id value. In the case of 'Other', it is defined by the source. This is the string sibling of enum attribute type_id.
    - Type ID (type_id) [Recommended]: Integer The type of software package.
    - Package UID (uid) [Optional]: A unique identifier for the package or library reported by the source tool. E.g., the libId within the sbom field of an OX Security Issue or the SPDX components.*.bom-ref.
    - Vendor Name (vendor_name) [Optional]: The name of the vendor who published the software package.
    - Version (version) [Required]: The software package version.

    """

    architecture: Optional[str] = None
    cpe_name: Optional[str] = None
    epoch: Optional[int] = None
    fixed_in_version: Optional[str] = None
    hash: Optional[FingerPrint] = None
    license: Optional[str] = None
    license_url: Optional[str] = None
    name: str
    package_manager: Optional[str] = None
    package_manager_url: Optional[str] = None
    path: Optional[str] = None
    purl: Optional[str] = None
    release: Optional[str] = None
    remediation: Optional[Remediation] = None
    src_url: Optional[str] = None
    type: Optional[str] = None
    type_id: Optional[SoftwarePackageTypeID] = None
    uid: Optional[str] = None
    vendor_name: Optional[str] = None
    version: str



from enum import IntEnum


class ClassUID(IntEnum):
    """
    The unique identifier of a class. A Class describes the attributes available in an event.

    2003 ComplianceFinding: Compliance Finding events describe results of evaluations performed against resources, to check compliance with various Industry Frameworks or Security Standards such as NIST SP 800-53, CIS AWS Foundations Benchmark v1.4.0, ISO/IEC 27001 etc. Note: if the event producer is a security control, the security_control profile should be applied and its attacks information, if present, should be duplicated into the finding_info object.
    Note: If the Finding is an incident, i.e. requires incident workflow, also apply the incident profile or aggregate this finding into an Incident Finding.
    """

    ComplianceFinding = 2003

    """
    The unique identifier of a class. A Class describes the attributes available in an event.

    2004 DetectionFinding: A Detection Finding describes detections or alerts generated by security products using correlation engines, detection engines or other methodologies. Note: if the product is a security control, the security_control profile should be applied and its attacks information should be duplicated into the finding_info object. Note: If the Finding is an incident, i.e. requires incident workflow, also apply the incident profile or aggregate this finding into an Incident Finding.
    """
    DetectionFinding = 2004

    """
    The unique identifier of a class. A Class describes the attributes available in an event.

    2007 ApplicationSecurityPostureFinding: The Application Security Posture Finding event is a notification about any bug, defect, deficiency, exploit, vulnerability, weakness or any other issue with software and related systems
    """
    ApplicationSecurityPostureFinding = 2007



from typing import Optional

from pydantic import BaseModel


class URL(BaseModel):
    """
    The Uniform Resource Locator (URL) object describes the characteristics of a URL.

    Attributes:
    """

    categories: Optional[list[str]] = None
    category_ids: Optional[list[int]] = None
    domain: Optional[str] = None
    hostname: Optional[str] = None
    path: Optional[str] = None
    port: Optional[int] = None
    query_string: Optional[str] = None
    resource_type: Optional[str] = None
    scheme: Optional[str] = None
    subdomain: Optional[str] = None
    url_string: Optional[str] = None



from enum import IntEnum
from typing import Optional

from pydantic import BaseModel


class TypeID(IntEnum):
    Unknown = 0
    Other = 99
    Windows = 100
    Windows_Mobile = 101
    Linux = 200
    Android = 201
    MacOS = 300
    IOS = 301
    IPadOS = 302
    Solaris = 400
    AIX = 401
    HP_UX = 402


class OperatingSystem(BaseModel):
    """
    Represents the operating system, detailing its architecture, country, language, name, build, and specific editions or service packs.
    It encompasses both broad categorization and specific identifiers like CPE names.

    Attributes:
    - CPU Bits (cpu_bits) [Optional]: The cpu architecture, the number of bits used for addressing in memory. For example: 32 or 64.
    - Country (country) [Optional]: The operating system country code, as defined by the ISO 3166-1 standard (Alpha-2 code). For the complete list of country codes, see ISO 3166-1 alpha-2 codes.
    - Language (lang) [Optional]: The two letter lower case language codes, as defined by ISO 639-1. For example: en (English), de (German), or fr (French).
    - Name (name) [Optional]: The operating system name.
    - OS Build (build) [Optional]: The operating system build number.
    - OS Edition (edition) [Optional]: The operating system edition. For example: Professional.
    - OS Service Pack Name (sp_name) [Optional]: The name of the latest Service Pack.
    - OS Service Pack Version (sp_ver) [Optional]: The version number of the latest Service Pack.
    - The product CPE identifier (cpe_name) [Optional]: The Common Platform Enumeration (CPE) name as described by (NIST) For example: cpe:/a:apple:safari:16.2.
    - Type (type) [Optional]: The type of the operating system.
    - Type ID (type_id) [Optional]: The type identifier of the operating system.
    - Version (version) [Optional]: The version of the OS running on the device that originated the event. For example: "Windows 10", "OS X 10.7", or "iOS 9".
    """

    cpu_bits: Optional[int] = None
    country: Optional[str] = None
    lang: Optional[str] = None
    name: str
    build: Optional[str] = None
    edition: Optional[str] = None
    sp_name: Optional[str] = None
    sp_ver: Optional[int] = None
    cpe_name: Optional[str] = None
    type: Optional[str] = None
    type_id: TypeID
    version: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.policy import Policy


class Assessment(BaseModel):
    """
    The Assessment object describes a point-in-time assessment, check, or evaluation of a specific configuration or signal against an asset, entity, person, or otherwise. For example, this can encapsulate os_signals from CrowdStrike Falcon Zero Trust Assessments, or account for Datastore configurations from Cyera, or capture details of Microsoft Intune configuration policies.

    Attributes:
    - Category (category) [Optional]: The Assessment object describes a point-in-time assessment, check, or evaluation of a specific configuration or signal against an asset, entity, person, or otherwise. For example, this can encapsulate os_signals from CrowdStrike Falcon Zero Trust Assessments, or account for Datastore configurations from Cyera, or capture details of Microsoft Intune configuration policies.
    - Description (desc) [Optional]: The description of the assessment criteria, or a description of the specific configuration or signal the assessment is targeting.
    - Meets Criteria (meets_criteria) [Optional]: Determines whether the assessment against the specific configuration or signal meets the assessments criteria. For example, if the assessment checks if a Datastore is encrypted or not, having encryption would be evaluated as true.
    - Name (name) [Optional]: The name of the configuration or signal being assessed. For example: Kernel Mode Code Integrity (KMCI) or publicAccessibilityState.
    - Policy (policy) [Optional]: The details of any policy associated with an assessment.
    - UID (uid) [Optional]: The unique identifier of the configuration or signal being assessed. For example: the signal_id.
    """

    category: Optional[str] = None
    desc: Optional[str] = None
    meets_criteria: Optional[bool] = None
    name: Optional[str] = None
    policy: Optional[Policy] = None
    uid: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.url import URL


class CWE(BaseModel):
    """
    Common Weakness Enumeration (CWE) details.
    """

    caption: Optional[str] = None
    src_url: Optional[URL] = None
    uid: str



from enum import IntEnum


class DetectionFindingTypeID(IntEnum):
    """
    The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: class_uid * 100 + activity_id.

    """

    Unknown = 200400
    Create = 200401
    Update = 200402
    Close = 200403
    Other = 200499



from typing import Optional

from pydantic import BaseModel


class Group(BaseModel):
    """
    The Group object represents a collection or association of entities, such as users, policies, or devices. It serves as a logical grouping mechanism to organize and manage entities with similar characteristics or permissions within a system or organization.

    Attributes:
    - Account Type (type) [Optional]: The type of the group or account.
    - Description (desc) [Optional]: The group description.
    - Domain (domain) [Optional]: The domain where the group is defined. For example: the LDAP or Active Directory domain.
    - Name (name) [Recommended]: The group name.
    - Privileges (privileges) [Optional]: The group privileges.
    - Unique ID	(uid) [Recommended]: TThe unique identifier of the group. For example, for Windows events this is the security identifier (SID) of the group.
    """

    type: Optional[str] = None
    desc: Optional[str] = None
    domain: Optional[str] = None
    name: Optional[str] = None
    privileges: Optional[list[str]] = None
    uid: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.group import Group
from py_ocsf_models.objects.user import User


class ResourceDetails(BaseModel):
    """
    The Resource Details object describes details about resources that were affected by the activity/event.

    Attributes:
    - Criticality (criticality) [Optional]: The criticality of the resource as defined by the event source.
    - Data (data) [Optional]: Additional data describing the resource.
    - Group (group) [Optional]: The name of the related resource group.
    - Labels (labels) [Optional]: The list of labels/tags associated to a resource.
    - Name (name) [Optional]: The name of the resource.
    - Namespace (namespace) [Optional]: The namespace is useful when similar entities exist that you need to keep separate.
    - Owner (owner) [Recommended]: The identity of the service or user account that owns the resource.
    - Type (type) [Optional]: The resource type as defined by the event source.
    - Unique ID (uid) [Optional]: The unique identifier of the resource.
    - Version (version) [Optional]: The version of the resource. For example 1.2.3.

    If Cloud profile:
    - Cloud Partition (cloud_partition) [Optional]: The canonical cloud partition name to which the region is assigned (e.g. AWS Partitions: aws, aws-cn, aws-us-gov).
    - Region (region) [Optional]: The cloud region of the resource.
    """

    cloud_partition: Optional[str] = None
    region: Optional[str] = None
    criticality: Optional[str] = None
    data: Optional[dict[str, object]] = None
    group: Optional[Group] = None
    labels: Optional[list[str]] = None
    name: Optional[str] = None
    namespace: Optional[str] = None
    owner: Optional[User] = None
    type: Optional[str] = None
    uid: Optional[str] = None
    version: Optional[str] = None



from typing import Optional

from pydantic import BaseModel


class Organization(BaseModel):
    """
    The Organization object describes characteristics of an organization or company and its division if any.

    Attributes:
    - Name (name): The name of the organization. For example, Widget, Inc.
    - Org Unit ID (ou_uid) [Optional]: The alternate identifier for an entity's unique identifier. For example, its Active Directory OU DN or AWS OU ID
    - Org Unit Name	(ou_name): The name of the organizational unit, within an organization. For example, Finance, IT, R&D
    - Unique ID	(uid): The unique identifier of the organization. For example, its Active Directory or AWS Org ID.
    """

    name: Optional[str] = None
    ou_uid: Optional[str] = None
    ou_name: Optional[str] = None
    uid: Optional[str] = None



from typing import Optional

from pydantic import BaseModel


class Analytic(BaseModel):
    """
    The Analytic object contains details about the analytic technique used to analyze and derive insights from the data or information that led to the creation of a finding or conclusion.

    Attributes:
    - Category (category) [Optional]: The analytic category.
    - Description (desc) [Optional]: The description of the analytic that generated the finding.
    - Name (name) [Recommended]: The name of the analytic that generated the finding.
    - Type (type) [Optional]: The analytic type.
    - Type ID (type_id) [Required]: The analytic type ID.
    - Unique ID (uid) [Recommended]: The unique identifier of the analytic that generated the finding.
    - Version (version) [Optional]: The analytic version. For example: 1.1.
    """

    category: Optional[str] = None
    desc: Optional[str] = None
    name: str
    type: Optional[str] = None
    type_id: int
    uid: str
    version: Optional[str] = None



from typing import Optional

from pydantic import BaseModel


class Technique(BaseModel):
    """
    The Technique object describes the technique ID and/or name associated to an attack, as defined by ATT&CK MatrixTM.


    Attributes:
    - Name (name) [Recommended]: The name of the attack technique, as defined by ATT&CK MatrixTM.
    - Source URL (src_url) [Optional]: The versioned permalink of the attack technique, as defined by ATT&CK MatrixTM.
    - Unique ID (uid) [Recommended]: The unique identifier of the attack technique, as defined by ATT&CK MatrixTM.
    """

    name: str
    src_url: Optional[str] = None
    uid: str


class Tactic(BaseModel):
    """
    The Tactic object describes the tactic ID and/or name that is associated to an attack, as defined by ATT&CK MatrixTM.


    Attributes:
    - Name (name) [Optional]: The tactic name that is associated with the attack technique, as defined by ATT&CK MatrixTM. For example: Reconnaissance.
    - Source URL (src_url) [Optional]: The versioned permalink of the attack tactic, as defined by ATT&CK MatrixTM. For example: https://attack.mitre.org/versions/v14/tactics/TA0043/.
    - Unique ID (uid) [Recommended]: The tactic ID that is associated with the attack technique, as defined by ATT&CK MatrixTM. For example: TA0043.
    """

    name: Optional[str] = None
    src_url: Optional[str] = None
    uid: Optional[str] = None


class SubTechnique(BaseModel):
    """
    The Sub Technique object describes the sub technique ID and/or name associated to an attack, as defined by ATT&CK MatrixTM.


    Attributes:
    - Name (name) [Optional]: The name of the attack sub technique, as defined by ATT&CK MatrixTM. For example: Scanning IP Blocks.
    - Source URL (src_url) [Optional]: The versioned permalink of the attack sub technique, as defined by ATT&CK MatrixTM. For example: https://attack.mitre.org/versions/v14/techniques/T1595/001/.
    - Unique ID (uid) [Recommended]: The unique identifier of the attack sub technique, as defined by ATT&CK MatrixTM. For example: T1595.001.
    """

    name: Optional[str] = None
    src_url: Optional[str] = None
    uid: Optional[str] = None


class MITREAttack(BaseModel):
    """
    The MITRE ATT&CK® object describes the tactic, technique & sub-technique associated to an attack as defined in ATT&CK MatrixTM.

    Attributes:
    - Sub Technique (sub_technique) [Optional]: The Sub Technique object describes the sub technique ID and/or name associated to an attack, as defined by ATT&CK MatrixTM.
    - Tactic (tactic) [Optional]: The Tactic object describes the tactic ID and/or name that is associated to an attack, as defined by ATT&CK MatrixTM.
    - Technique (technique) [Optional]: The Technique object describes the technique ID and/or name associated to an attack, as defined by ATT&CK MatrixTM.
    - Version (version) [Recommended]: The ATT&CK MatrixTM version.
    """

    sub_technique: Optional[SubTechnique] = None
    tactic: Optional[Tactic] = None
    technique: Optional[Technique] = None
    version: Optional[str] = None



from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.events.findings.severity_id import SeverityID
from py_ocsf_models.objects.compliance_status import StatusID


class Check(BaseModel):
    """
    The check object defines a specific, testable compliance verification point that evaluates a target device against a standard, framework, or custom requirement. While checks are typically associated with formal standards (like CIS, NIST, or ISO), they can also represent custom or organizational requirements. When mapped to controls, checks can evaluate specific control_parameters to determine compliance status, but neither the control mapping nor control_parameters are required for a valid check.

    Attributes:
    - Description (desc) [Optional]: The detailed description of the compliance check, explaining the security requirement, vulnerability, or configuration being assessed. For example, CIS: The cramfs filesystem type is a compressed read-only Linux filesystem. Removing support for unneeded filesystem types reduces the local attack surface. or DISA STIG: Unauthorized access to the information system by foreign entities may result in loss or compromise of data.
    - Name (name) [Optional]: The name or title of the compliance check. For example, CIS: Ensure mounting of cramfs filesystems is disabled or DISA STIG: The Ubuntu operating system must implement DoD-approved encryption to protect the confidentiality of remote access sessions.

    - Severity (severity) [Optional]: The severity level as defined in the source document. For example CIS Benchmarks, valid values are: Level 1 (security-forward, essential settings), Level 2 (security-focused environment, more restrictive), or Scored/Not Scored (whether compliance can be automatically checked). For DISA STIG, valid values are: CAT I (maps to severity_id 5/Critical), CAT II (maps to severity_id 4/High), or CAT III (maps to severity_id 3/Medium).
    - Severity ID (severity_id) [Optional]: The normalized severity identifier that maps severity levels to standard severity levels. For example CIS Benchmark: Level 2 maps to 4 (High), Level 1 maps to 3 (Medium). For DISA STIG: CAT I maps to 5 (Critical), CAT II maps to 4 (High), and CAT III maps to 3 (Medium).
    - Standards (standards) [Optional]: The regulatory or industry standard this check is associated with. E.g., PCI DSS 3.2.1, HIPAA Security Rule, NIST SP 800-53 Rev. 5, or ISO/IEC 27001:2013.
    - Status (status) [Optional]: The resultant status of the compliance check normalized to the caption of the status_id value. For example, CIS Benchmark: Pass when all requirements are met, Fail when requirements are not met, or DISA STIG: NotAFinding (maps to status_id 1/Pass), Open (maps to status_id 3/Fail).
    - Status ID (status_id) [Optional]: The normalized status identifier of the compliance check.
    - UID (uid) [Optional]: The unique identifier of the compliance check within its standard or framework. For example, CIS Benchmark identifier 1.1.1.1, DISA STIG identifier V-230234, or NIST control identifier AC-17(2).
    - Version (version) [Optional]: The check version. For example, CIS Benchmark: 1.1.0 for Amazon Linux 2 or DISA STIG: V2R1 for Windows 10.
    """

    desc: Optional[str] = None
    name: Optional[str] = None
    severity: Optional[str] = None
    severity_id: Optional[SeverityID] = None
    standards: Optional[List[str]] = None
    status: Optional[str] = None
    status_id: Optional[StatusID] = None
    uid: Optional[str] = None
    version: Optional[str] = None



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.objects.cvss import CVSSScore
from py_ocsf_models.objects.cwe import CWE
from py_ocsf_models.objects.epss import EPSS
from py_ocsf_models.objects.product import Product


class CVE(BaseModel):
    """
    Common Vulnerabilities and Exposures (CVE) details.
    """

    created_time: Optional[datetime] = None
    cvss: Optional[list[CVSSScore]] = None
    desc: Optional[str] = None
    epss: Optional[EPSS] = None
    modified_time: Optional[datetime] = None
    product: Optional[Product] = None
    references: Optional[List[str]] = None
    related_cwes: Optional[List[CWE]] = None
    title: Optional[str] = None
    type: Optional[str] = None
    uid: str



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.group import Group
from py_ocsf_models.objects.request_elements import RequestElements
from py_ocsf_models.objects.response_elements import ResponseElements
from py_ocsf_models.objects.service import Service


class API(BaseModel):
    """
    Represents the details of an API interaction, including both request and response elements,
    as well as metadata about the API's operation and the service it belongs to.

    Attributes:
    - request (Optional[RequestElements]): Details pertaining to the API request.
    - response (Optional[ResponseElements]): Details pertaining to the API response.
    - group (Optional[Group]): Information about the API group, if applicable.
    - operation (str): The HTTP verb or operation associated with the API request, such as GET, POST, PUT, DELETE.
    - service (Optional[Service]): Information about the API service, including its name and other relevant details.
    - version (Optional[str]): The version of the API, indicating the specific iteration of the API service being used.
    """

    request: Optional[RequestElements] = None
    response: Optional[ResponseElements] = None
    group: Optional[Group] = None
    operation: str
    service: Optional[Service] = None
    version: Optional[str] = None



from typing import Optional

from pydantic import BaseModel


class GeoLocation(BaseModel):
    """
    The GeoLocation object encapsulates geographical information about a specific location. This includes details that can be used to pinpoint a location globally, identify regional characteristics, and provide additional context about the environment of the location.

    Attributes:
    - City (city) [Recommended]: Name of the city for the location.
    - Continent (continent) [Recommended]: Name of the continent where the location is situated.
    - Coordinates (coordinates) [Recommended]: A longitude/latitude pair conforming to GeoJSON format.
    - Country (country) [Recommended]: ISO 3166-1 Alpha-2 country code, capitalized.
    - Description (desc) [Optional]: A description of the geographical location.
    - ISP (isp) [Optional]: Internet Service Provider associated with the location.
    - On Premises (is_on_premises) [Optional]: Indicates if the location is on premises.
    - Postal Code (postal_code) [Optional]: Postal code for the location.
    - Provider (provider) [Optional]: Provider of the geographical data.
    - Region (region) [Optional]: The principal subdivision of the country, such as a state or province.
    """

    city: Optional[str] = None
    continent: Optional[str] = None
    coordinates: Optional[list[float]] = None
    country: Optional[str] = None
    desc: Optional[str] = None
    isp: Optional[str] = None
    is_on_premises: Optional[bool] = None
    postal_code: Optional[str] = None
    provider: Optional[str] = None
    region: Optional[str] = None



from enum import IntEnum


class StatusID(IntEnum):
    """
    The normalized status identifier of the Finding, set by the consumer.

    0 Unknown: The status is unknown.
    1 New: The Finding is new and yet to be reviewed.
    2 InProgress: The Finding is under review.
    3 Suppressed: The Finding was reviewed, determined to be benign or a false positive and is now suppressed.
    4 Resolved: The Finding was reviewed, remediated and is now considered resolved.
    5 Archived: The Finding was archived.
    99 Other: The event status is not mapped. See the status attribute, which contains a data source specific value.
    """

    Unknown = 0
    New = 1
    InProgress = 2
    Suppressed = 3
    Resolved = 4
    Archived = 5
    Other = 99



from enum import IntEnum


class ImpactID(IntEnum):
    """
    The normalized impact of the finding.

    0 Unknown: The normalized impact is unknown.
    1 Low
    2 Medium
    3 High
    4 Critical
    99 Other: The impact is not mapped. See the impact attribute, which contains a data source specific value.
    """

    Unknown = 0
    Low = 1
    Medium = 2
    Hig = 3
    Critical = 4
    Other = 99



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.objects.affected_software_package import AffectedSoftwarePackage
from py_ocsf_models.objects.cve import CVE
from py_ocsf_models.objects.cwe import CWE
from py_ocsf_models.objects.remediation import Remediation


class VulnerabilityDetails(BaseModel):
    """
    The vulnerability is an unintended characteristic of a computing component or system configuration that multiplies the risk of an adverse event or a loss occurring either due to accidental exposure, deliberate attack, or conflict with new system components.

    Attributes:
    - Affected Code (affected_code) [Optional]: List of AffectedCode objects that describe details about code blocks identified as vulnerable.
    - Affected Software Packages (affected_packages) [Optional]: List of software packages identified as affected by a vulnerability/vulnerabilities.
    - CVE (cve) [Recommended]: The Common Vulnerabilities and Exposures (CVE).
    - CWE (cwe) [Recommended]: The CWE object represents a weakness in a software system that can be exploited by a threat actor to perform an attack.
    - Description (desc) [Optional]: The description of the vulnerability.
    - Exploit Availability (is_exploit_available) [Optional]: Indicates if an exploit or a PoC (proof-of-concept) is available for the reported vulnerability.
    - First Seen (first_seen_time) [Optional]: The time when the vulnerability was first observed.
    - First Seen (first_seen_time_dt) [Optional]: The time when the vulnerability was first observed in datetime format.
    - Knowledgebase Articles (kb_article_list) [Optional]: A list of KB articles or patches related to an endpoint. A KB Article contains metadata that describes the patch or an update.
    - Last Seen (last_seen_time) [Optional]: The time when the vulnerability was most recently observed.
    - Last Seen (last_seen_time_dt) [Optional]: The time when the vulnerability was most recently observed in datetime format.
    - References (references) [Recommended]: A list of reference URLs with additional information about the vulnerability.
    - Related Vulnerabilities (related_vulnerabilities) [Optional]: List of vulnerabilities that are related to this vulnerability.
    - Remediation Guidance (remediation) [Optional]: The remediation recommendations on how to mitigate the identified vulnerability.
    - Severity (severity) [Optional]: The vendor assigned severity of the vulnerability.
    - Title (title) [Optional]: A title or a brief phrase summarizing the discovered vulnerability.
    - Vendor Name (vendor_name) [Optional]: The name of the vendor that identified the vulnerability.

    Just one of these mutually exclusive attributes must be present: advisory, cve, cwe

    """

    # TODO
    # advisory: Optional[Advisory]
    # affected_code: Optional[List[AffectedCode]]
    affected_packages: Optional[List[AffectedSoftwarePackage]] = None
    cve: Optional[CVE] = None
    cwe: Optional[CWE] = None
    desc: Optional[str] = None
    is_exploit_available: Optional[bool] = None
    first_seen_time: Optional[int] = None
    first_seen_time_dt: Optional[datetime] = None
    last_seen_time: Optional[int] = None
    last_seen_time_dt: Optional[datetime] = None
    references: Optional[List[str]] = None
    related_vulnerabilities: Optional[List[str]] = None
    remediation: Optional[Remediation] = None
    severity: Optional[str] = None
    title: Optional[str] = None
    vendor_name: Optional[str] = None



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class EPSS(BaseModel):
    """
    The Exploit Prediction Scoring System (EPSS) object describes the estimated probability a vulnerability will be exploited. EPSS is a community-driven effort to combine descriptive information about vulnerabilities (CVEs) with evidence of actual exploitation in-the-wild. (EPSS).
    """

    created_time: Optional[datetime] = None
    percentile: Optional[float] = None
    score: str
    version: Optional[str] = None



from enum import IntEnum


class ActivityID(IntEnum):
    """
    The normalized identifier of the finding activity.

    0 Unknown: The event activity is unknown.
    1 Create: A finding was created.
    2 Update: A finding was updated.
    3 Close: A finding was closed.
    99 Other: The event activity is not mapped. See the activity_name attribute, which contains a data source specific value.
    """

    Unknown = 0
    Create = 1
    Update = 2
    Close = 3
    Other = 99



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.events.base_event import BaseEvent
from py_ocsf_models.events.findings.activity_id import ActivityID
from py_ocsf_models.events.findings.confidence_id import ConfidenceID
from py_ocsf_models.objects.finding_info import FindingInformation


class Finding(BaseEvent, BaseModel):
    """
    The Finding event is a generic event that defines a set of attributes available in the Findings category.

    Attributes:
    - Activity (activity_name) [Optional]: The finding activity name, as defined by the activity_id.
    - Activity ID (activity_id): The normalized identifier of the finding activity.
    - Comment (comment) [Optional]: A user provided comment about the finding.
    - Confidence (confidence) [Optional]: The confidence, normalized to the caption of the confidence_id value. In the case of 'Other', it is defined by the event source.
    - Confidence ID (confidence_id) [Optional]: Represents the accuracy of the detection rule. A low confidence indicates a broad finding scope that may include benign events.
    - Confidence Score (confidence_score) [Optional]: The confidence score as reported by the event source.
    - End Time (end_time) [Optional]: Time of the latest event included in the finding.
    - End Time DT (end_time_dt) [Optional]: Time of the latest event included in the finding in datetime format.
    - Finding Information (finding_info) [Required]: Describes the supporting information about a generated finding.
    - Start Time (start_time) [Optional]: Time of the earliest event included in the finding.
    - Start Time DT (start_time_dt) [Optional]: Time of the earliest event included in the finding in datetime

    """

    activity_name: Optional[str] = None
    activity_id: ActivityID
    comment: Optional[str] = None
    confidence: Optional[str] = None
    confidence_id: Optional[ConfidenceID] = None
    confidence_score: Optional[int] = None
    end_time: Optional[int] = None
    end_time_dt: Optional[datetime] = None
    finding_info: FindingInformation
    start_time: Optional[int] = None
    start_time_dt: Optional[datetime] = None



from enum import IntEnum


class RiskLevelID(IntEnum):
    """
    The normalized risk level id.

    0 Info
    1 Low
    2 Medium
    3 High
    4 Critical
    """

    Info = 0
    Low = 1
    Medium = 2
    High = 3
    Critical = 4



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.operating_system import OperatingSystem
from py_ocsf_models.objects.product import Product


class KBArticle(BaseModel):
    """
    Describes a knowledgebase article, providing essential information such as its classification,
    release date, applicable operating system, and severity. It includes details like the article's
    size, source URL, and whether it has been superseded by another patch.

    Attributes:
    - classification: Vendor's classification of the KB article.
    - created_time: Time the KB article was created.
    - created_time_dt: Time the KB article was created in datetime
    - os: Operating system the KB article applies to.
    - bulletin: Bulletin identifier of the KB article.
    - product: Product details the KB article applies to.
    - severity: Severity rating of the KB article.
    - size: Size of the KB article in bytes.
    - src_url: Link to the KB article from the vendor.
    - is_superseded: Indicates if the article has been replaced by another.
    - title: Title of the KB article.
    - uid: Unique identifier for the KB article.
    """

    classification: Optional[str] = None
    created_time: Optional[int] = None
    created_time_dt: Optional[datetime] = None
    os: OperatingSystem
    bulletin: Optional[str] = None
    product: Optional[Product] = None
    severity: str
    size: Optional[int] = None
    src_url: Optional[str] = None
    is_superseded: Optional[bool] = None
    title: str
    uid: str



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.account import Account
from py_ocsf_models.objects.organization import Organization


class Cloud(BaseModel):
    """
    The Cloud object contains information about a cloud account such as AWS Account ID, regions, etc.

    Attributes:
    - Account (account) [Optional]: The account object describes details about the account that was the source or target of the activity.
    - Network Zone (zone) [Optional]: The availability zone in the cloud region, as defined by the cloud provider.
    - Organization (org) [Optional]: Organization and org unit relevant to the event or object.
    - Project ID (project_uid) [Optional]: The unique identifier of a Cloud project.
    - Provider (provider) [Required]: The unique name of the Cloud services provider, such as AWS, MS Azure, GCP, etc.
    - Region (region) [Recommended]: The name of the cloud region, as defined by the cloud provider.
    """

    account: Optional[Account] = None
    zone: Optional[str] = None
    org: Optional[Organization] = None
    provider: str
    region: Optional[str] = None



from enum import IntEnum
from typing import Optional

from pydantic import BaseModel


class AlgorithmID(IntEnum):
    """
    The identifier of the normalized hash algorithm, which was used to create the digital fingerprint.

    0 Unknown: The algorithm is unknown.
    1 MD5: MD5 message-digest algorithm producing a 128-bit (16-byte) hash value.
    2 SHA-1: Secure Hash Algorithm 1 producing a 160-bit (20-byte) hash value.
    3 SHA-256: Secure Hash Algorithm 2 producing a 256-bit (32-byte) hash value.
    4 SHA-512: Secure Hash Algorithm 2 producing a 512-bit (64-byte) hash value.
    5 CTPH: The ssdeep generated fuzzy checksum. Also known as Context Triggered Piecewise Hash (CTPH).
    6 TLSH: The TLSH fuzzy hashing algorithm.
    7 quickXorHash: Microsoft simple non-cryptographic hash algorithm that works by XORing the bytes in a circular-shifting fashion.
    99 Other: The algorithm is not mapped. See the algorithm attribute, which contains a data source specific value.

    """

    Unknown = 0
    MD5 = 1
    SHA_1 = 2
    SHA_256 = 3
    SHA_512 = 4
    CTPH = 5
    TLSH = 6
    QuickXorHash = 7
    Other = 99


class FingerPrint(BaseModel):
    """
    Represents a digital fingerprint created using a specific hashing algorithm. This class
    encapsulates details about the algorithm used and the resulting hash value.

    Attributes:
    - Algorithm (algorithm) [Optional]: The hash algorithm used to create the digital fingerprint, normalized to the caption of 'algorithm_id'. In the case of 'Other', it is defined by the event source.
    - Algorithm ID (algorithm_id): The identifier of the normalized hash algorithm, which was used to create the digital fingerprint.
    - Value (value): The digital fingerprint value.
    """

    algorithm: Optional[str] = None
    algorithm_id: AlgorithmID
    value: str



from typing import Optional

from pydantic import BaseModel


class Enrichment(BaseModel):
    """
    Represents enrichment data associated with a specific attribute, providing additional context or information.
    This can include various types of data, depending on the enrichment type, such as location data for an IP address.

    Attributes:
    - data (dict): The JSON-formatted enrichment data, whose structure depends on the enrichment type.
    - name (str): The name of the attribute being enriched.
    - provider (str): The name of the provider supplying the enrichment data.
    - type (str): The type of enrichment, indicating the nature of the data provided.
    - value (str): The value of the attribute to which the enrichment data pertains.
    """

    data: dict[str, object]
    name: str
    provider: Optional[str] = None
    type: Optional[str] = None
    value: str



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.metric import Metric
from py_ocsf_models.objects.url import URL


class CVSSScore(BaseModel):
    """
    Common Vulnerability Scoring System (CVSS) details.
    """

    base_score: float
    depth: Optional[str] = None
    metrics: Optional[list[Metric]] = None
    overall_score: Optional[float] = None
    severity: Optional[str] = None
    src_url: Optional[URL] = None
    vector_string: Optional[str] = None
    vendor_name: Optional[str] = None
    version: Optional[str] = None



from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models.objects.assessment import Assessment
from py_ocsf_models.objects.check import Check
from py_ocsf_models.objects.compliance_status import StatusID


class Compliance(BaseModel):
    """
    The Compliance object contains information about Industry and Regulatory Framework standards, controls and requirements or details about custom assessments utilized in a compliance evaluation. Standards define broad security frameworks, controls represent specific security requirements within those frameworks, and checks are the testable verification points used to determine if controls are properly implemented.

    Attributes:
    - Assessments (assessments) [Optional]: A list of assessments associated with the compliance requirements evaluation.
    - Category (category) [Optional]: The category a control framework pertains to, as reported by the source tool, such as Asset Management or Risk Assessment.
    - Checks (checks) [Optional]: A list of compliance checks associated with specific industry standards or frameworks. Each check represents an individual rule or requirement that has been evaluated against a target device. Checks typically include details such as the check name (e.g., CIS: 'Ensure mounting of cramfs filesystems is disabled' or DISA STIG descriptive titles), unique identifiers (such as CIS identifier '1.1.1.1' or DISA STIG identifier 'V-230234'), descriptions (detailed explanations of security requirements or vulnerability discussions), and version information.
    - Control (control) [Optional]: A Control is a prescriptive, actionable set of specifications that strengthens device posture. The control specifies required security measures, while the specific implementation values are defined in control_parameters. E.g., CIS AWS Foundations Benchmark 1.2.0 - Control 2.1 - Ensure CloudTrail is enabled in all regions
    - Description (desc) [Optional]: The description or criteria of a control.
    - Requirements (requirements) [Optional]: The specific compliance requirements being evaluated. E.g., PCI DSS Requirement 8.2.3 - Passwords must meet minimum complexity requirements or HIPAA Security Rule 164.312(a)(2)(iv) - Implement encryption and decryption mechanisms
    - Standards (standards) [Optional]: The regulatory or industry standards being evaluated for compliance.
    """

    assessments: Optional[List[Assessment]] = None
    category: Optional[str] = None
    checks: Optional[List[Check]] = None
    control: Optional[str] = None
    desc: Optional[str] = None
    requirements: Optional[List[str]] = None
    standards: Optional[List[str]] = None
    status_id: Optional[StatusID] = None



from typing import Optional

from pydantic import BaseModel


class Policy(BaseModel):
    """
    The Policy object describes the policies that are applicable. Policy attributes provide traceability to the operational state of the security product at the time that the event was captured, facilitating forensics, troubleshooting, and policy tuning/adjustments.

    Attributes:
    - Data (data) [Optional]: Additional data about the policy such as the underlying JSON policy itself or other details.
    - Description (desc) [Optional]: The description of the policy.
    - Group (group) [Optional]: The policy group.
    - Is Applied (is_applied) [Optional]: A determination if the content of a policy was applied to a target or request, or not.
    - Name (name) [Optional]: The policy name. For example: IAM Policy.
    - UID (uid) [Optional]: A unique identifier of the policy instance.
    - Version (version) [Optional]: The policy version number.
    """

    data: Optional[str] = None
    desc: Optional[str] = None
    group: Optional[str] = None
    is_applied: Optional[bool] = None
    name: Optional[str] = None
    uid: Optional[str] = None
    version: Optional[str] = None



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel

from py_ocsf_models import OCSF_VERSION
from py_ocsf_models.objects.device import Device
from py_ocsf_models.objects.product import Product


class SchemaExtension(BaseModel):
    """
    The OCSF Schema Extension object provides detailed information about the schema extension used to construct the event.

    Attributes:
    - Name (name) [Required]: The schema extension name (e.g., dev).
    - Unique ID (uid) [Required]: The unique identifier of the schema extension (e.g., 999).
    - Version (version) [Required]: The version of the schema extension (e.g., 1.0.0-alpha.2).
    """

    name: str
    uid: str
    version: str


class Logger(BaseModel):
    """
    The Logger object represents the device and product where events are stored with times for receipt and transmission. This may be at the source device where the event occurred, a remote scanning device, intermediate hops, or the ultimate destination.

    Attributes:
    - Device (device) [Recommended]: The device where the events are logged.
    - Log Level (log_level) [Optional]: The audit level at which an event was generated.
    - Log Name (log_name) [Recommended]: The event log name (e.g., syslog file name or Windows logging subsystem: Security).
    - Log Provider (log_provider) [Recommended]: The logging provider or logging service that logged the event (e.g., Microsoft-Windows-Security-Auditing).
    - Log Version (log_version) [Optional]: The event log schema version that specifies the format of the original event.
    - Logged Time (logged_time) [Optional]: The time when the logging system collected and logged the event.
    - Name (name) [Recommended]: The name of the logging product instance.
    - Product (product) [Recommended]: The product logging the event (e.g., event source product, management server product, SIEM, etc.).
    - Transmission Time (transmit_time) [Optional]: The time when the event was transmitted from the logging device to its next destination.
    - Unique ID (uid) [Recommended]: The unique identifier of the logging product instance.
    - Version (version) [Optional]: The version of the logging product.
    """

    device: Optional[Device] = None
    log_level: Optional[str] = None
    log_name: str
    log_provider: str
    log_version: Optional[str] = None
    logged_time: Optional[datetime] = None
    name: str
    product: Product
    transmit_time: Optional[datetime] = None
    uid: str
    version: Optional[str] = None


class Metadata(BaseModel):
    """
    The Metadata object describes the metadata associated with the event.

    Attributes:
    - Correlation UID (correlation_uid) [Optional]: Unique identifier used to correlate events.
    - Event Code (event_code) [Optional]: The Event ID or Code describing the event.
    - Event UID (uid) [Optional]: Logging system-assigned unique identifier of an event instance.
    - Labels (labels) [Optional]: List of category labels attached to the event or specific attributes.
    - Log Level (log_level) [Optional]: Audit level at which the event was generated.
    - Log Name (log_name) [Optional]: Event log name (e.g., syslog file name or Windows logging subsystem).
    - Log Provider (log_provider) [Optional]: Logging provider or service that logged the event.
    - Log Version (log_version) [Optional]: Event log schema version specifying the format of the original event.
    - Logged Time (logged_time) [Optional]: Time when the logging system collected and logged the event.
    - Loggers (loggers) [Optional]: Array of Logger objects describing the devices and logging products in the event flow.
    - Modified Time (modified_time) [Optional]: Time when the event was last modified or enriched.
    - Original Time (original_time) [Optional]: Original event time as reported by the event source.
    - Processed Time (processed_time) [Optional]: Event processed time, such as during an ETL operation.
    - Product (product) [Required]: Product that reported the event.
    - Profiles (profiles) [Optional]: List of profiles used to create the event.
    - Extensions (extensions) [Optional]: Schema extensions used to create the event.
    - Sequence (sequence) [Optional]: Sequence number of the event for unambiguous ordering.
    - Tenant UID (tenant_uid) [Optional]: Unique tenant identifier.
    - Version (version) [Required]: Version of the OCSF schema in Semantic Versioning Specification (SemVer).
    """

    correlation_uid: Optional[str] = None
    event_code: Optional[str] = None
    uid: Optional[str] = None
    labels: Optional[List[str]] = None
    log_level: Optional[str] = None
    log_name: Optional[str] = None
    log_provider: Optional[str] = None
    log_version: Optional[str] = None
    logged_time: Optional[datetime] = None
    loggers: Optional[List[Logger]] = None
    modified_time: Optional[datetime] = None
    original_time: Optional[str] = None
    processed_time: Optional[datetime] = None
    product: Product
    profiles: Optional[List[str]] = None
    extensions: Optional[List[SchemaExtension]] = None
    sequence: Optional[int] = None
    tenant_uid: Optional[str] = None
    version: str = OCSF_VERSION



from enum import IntEnum
from typing import Optional

from pydantic import BaseModel


class DNSOpcodeID(IntEnum):
    """
    Enum representing different DNS opcode identifiers.

    Attributes:
    - Query (0): Standard query.
    - Inverse Query (1): Inverse query, obsolete.
    - Status (2): Server status request.
    - Reserved (3): Reserved, not used.
    - Notify (4): Zone change notification.
    - Update (5): Dynamic DNS update.
    - DSO Message (6): DNS Stateful Operations (DSO).
    """

    Query = 0
    Inverse_Query = 1
    Status = 2
    Reserved = 3
    Notify = 4
    Update = 5
    DSO_Message = 6


class DNSQuery(BaseModel):
    """
    The DNS query object represents a specific request made to the Domain Name System (DNS) to retrieve information about a domain or perform a DNS operation.
    This object encapsulates the necessary attributes and methods to construct and send DNS queries, specify the query type (e.g., A, AAAA, MX).

    Attributes:
    - DNS Opcode (opcode) [Optional]: The DNS opcode specifies the type of the query message.
    - DNS Opcode ID (opcode_id) [Recommended]: The DNS opcode ID specifies the normalized query message type.
    - Hostname (hostname) [Required]: The hostname or domain being queried. For example: www.example.com.
    - Packet UID (packet_uid) [Recommended]: The DNS packet identifier assigned by the program that generated the query. The identifier is copied to the response.
    - Resource Record Class (class) [Recommended]: The class of resource records being queried. See RFC1035. For example: IN.
    - Resource Record Type (type) [Recommended]: The type of resource records being queried. See RFC1035. For example: A, AAAA, CNAME, MX, and NS.
    """

    opcode: Optional[str] = None
    opcode_id: Optional[DNSOpcodeID] = None
    hostname: str
    packet_uid: Optional[int] = None
    # TODO: Update to Pydantic v2 to use Field
    # class_: Optional[str] = Field(
    #     alias="class"
    # )  # Renaming class to avoid conflict with Python keyword
    type: Optional[str] = None



from datetime import datetime
from typing import Optional

# FIXME(circular-dependency)
# from py_ocsf_models.objects.user import User
from pydantic import BaseModel, EmailStr

from py_ocsf_models.objects.geolocation import GeoLocation


class LDAPPerson(BaseModel):
    """
    The LDAPPerson class encapsulates detailed information about an individual within an LDAP or Active Directory system. It is designed to model both the professional and personal attributes of a person, including their role, contact information, and organizational context.

    Attributes:
    - Cost Center (cost_center) [Optional]: Associated cost center for budgeting and financial analysis.
    - Created Time (created_time) [Optional]: Timestamp of when the user account was created.
    - Deleted Time (deleted_time) [Optional]: Timestamp indicating when the user was deleted, useful in AD environments.
    - Email Addresses (email_addrs) [Optional]: list of additional email addresses.
    - Employee ID (employee_uid) [Optional]: Unique identifier assigned by the organization.
    - Geo Location (location) [Optional]: Usual work location of the user.
    - Given Name (given_name) [Optional]: First name of the user.
    - Hire Time (hire_time) [Optional]: Timestamp when the user was hired.
    - Job Title (job_title) [Optional]: Official job title.
    - LDAP Common Name (ldap_cn) [Optional]: Full name as per LDAP cn attribute.
    - LDAP Distinguished Name (ldap_dn) [Optional]: Unique DN in the LDAP directory.
    - Labels (labels) [Optional]: Array of labels or tags associated with the user.
    - Last Login (last_login_time) [Optional]: Last login timestamp.
    - Leave Time (leave_time) [Optional]: When the user left or will leave the organization.
    - Manager (manager) [Optional]: Direct manager, supports org hierarchy.
    - Modified Time (modified_time) [Optional]: Last modification timestamp of the user entry.
    - Office Location (office_location) [Optional]: Primary office location, not necessarily a specific address.
    - Surname (surname) [Optional]: Family or last name.
    """

    cost_center: Optional[str] = None
    created_time: Optional[datetime] = None
    deleted_time: Optional[datetime] = None
    email_addrs: Optional[list[EmailStr]] = None
    employee_uid: Optional[str] = None
    location: Optional[GeoLocation] = None
    given_name: Optional[str] = None
    hire_time: Optional[datetime] = None
    job_title: Optional[str] = None
    ldap_cn: Optional[str] = None
    ldap_dn: Optional[str] = None
    labels: Optional[list[str]] = None
    last_login_time: Optional[datetime] = None
    leave_time: Optional[datetime] = None
    # FIXME(circular-dependency)
    # manager: Optional[User]
    modified_time: Optional[datetime] = None
    office_location: Optional[str] = None
    surname: Optional[str] = None



from enum import IntEnum


class StatusID(IntEnum):
    """
    The normalized status identifier of the compliance check.

    0   Unknown: The status is unknown.
    1   Pass: The compliance check passed for all the evaluated resources.
    2   Warning: The compliance check did not yield a result due to missing information.
    3   Fail: The compliance check failed for at least one of the evaluated resources.
    99	Other: The event status is not mapped. See the status attribute, which contains a data source specific value.
    """

    Unknown = 0
    Pass = 1
    Warning = 2
    Fail = 3
    Other = 99



from typing import Optional

from pydantic import BaseModel


class Image(BaseModel):
    """
    Represents a container image, detailing its name, optional tag, labels, and unique identifier.
    The class encapsulates the core attributes that define a container image in a containerized environment.

    Attributes:
    - tag (Optional[str]): The image tag, specifying version or variant, e.g., '1.11-alpine'.
    - labels (Optional[list[str]]): Metadata labels associated with the image.
    - name (str): The name of the image, e.g., 'elixir'.
    - path (Optional[str]): The full path to the image file on the host or in the repository.
    - uid (str): A unique identifier for the image, e.g., '77af4d6b9913'.
    """

    tag: Optional[str] = None
    labels: Optional[list[str]] = None
    name: str
    path: Optional[str] = None
    uid: str



from typing import Optional

from pydantic import BaseModel


class Service(BaseModel):
    """
    Encapsulates information about a service, including its name, unique identifier, version, and any associated labels.
    This model is designed to represent services in a microservices architecture or within any application ecosystem, providing
    a way to uniquely identify and describe services.

    Attributes:
    - Labels (labels) [Optional]: A list of labels or tags associated with the service, providing additional metadata.
    - Name (name) [Optional]: The name of the service, clearly identifying it within the system.
    - Unique ID	 (uid) [Optional]: A unique identifier for the service, ensuring distinct recognition.
    - Version (version) [Optional]: The version of the service, helping to track changes or updates over time.
    """

    labels: Optional[list[str]] = None
    name: str
    uid: str
    version: str



from enum import IntEnum

from pydantic import BaseModel


class PhaseID(IntEnum):
    """
    The Kill Chain Phase object represents a single phase of a cyber attack, including the initial reconnaissance and planning stages up to the final objective of the attacker. It provides a detailed description of each phase and its associated activities within the broader context of a cyber attack.

    Attributes:
    - Unknown (0): The kill chain phase is unknown.
    - Reconnaissance (1): The attackers pick a target and perform a detailed analysis, start collecting information (email addresses, conferences information, etc.) and evaluate the victim’s vulnerabilities to determine how to exploit them.
    - Weaponization (2): The attackers develop a malware weapon and aim to exploit the discovered vulnerabilities.
    - Delivery (3): The intruders will use various tactics, such as phishing, infected USB drives, etc.
    - Exploitation (4): The intruders start leveraging vulnerabilities to executed code on the victim’s system.
    - Installation (5): The intruders install malware on the victim’s system.
    - Command & Control (6): Malware opens a command channel to enable the intruders to remotely manipulate the victim's system.
    - Actions on Objectives (7): With hands-on keyboard access, intruders accomplish the mission’s goal.
    - Other (99): The kill chain phase is not mapped. See the phase attribute, which contains a data source specific value.
    """

    Unknown = 0
    Reconnaissance = 1
    Weaponization = 2
    Delivery = 3
    Exploitation = 4
    Installation = 5
    Command_Control = 6
    Actions_on_Objectives = 7
    Other = 99


class KillChainPhase(BaseModel):
    """
    Base model representing a Cyber Kill Chain phase.

    Attributes:
    - Kill Chain Phase (phase) [Recommended]: The cyber kill chain phase.
    - Kill Chain Phase ID (phase_id) [Required]: The cyber kill chain phase identifier.
    """

    phase: str
    phase_id: PhaseID



from typing import Optional

from pydantic import BaseModel


class KeyboardInformation(BaseModel):
    """
    The Keyboard Information object contains details and attributes related to a computer or device keyboard. It encompasses information that describes the characteristics, capabilities, and configuration of the keyboard.

    Attributes:
    - Function Keys (function_keys) [Optional]: The number of function keys on the client keyboard.
    - IME (ime) [Optional]: The Input Method Editor (IME) file name.
    - Keyboard Layout (keyboard_layout) [Optional]: The keyboard locale identifier name (e.g., en-US).
    - Keyboard Subtype (keyboard_subtype) [Optional]: The keyboard numeric code.
    - Keyboard Type (keyboard_type) [Optional]: The keyboard type (e.g., xt, ico).
    """

    function_keys: Optional[int] = None
    ime: Optional[str] = None
    keyboard_layout: Optional[str] = None
    keyboard_subtype: Optional[int] = None
    keyboard_type: Optional[str] = None


class Display(BaseModel):
    """
    The Display object contains information about the physical or virtual display connected to a computer system.

    Attributes:
    - Color Depth (color_depth) [Optional]: The numeric color depth.
    - Physical Height (physical_height) [Optional]: The numeric physical height of the display.
    - Physical Orientation (physical_orientation) [Optional]: The numeric physical orientation of the display.
    - Physical Width (physical_width) [Optional]: The numeric physical width of the display.
    - Scale Factor (scale_factor) [Optional]: The numeric scale factor of the display.
    """

    color_depth: Optional[int] = None
    physical_height: Optional[int] = None
    physical_orientation: Optional[int] = None
    physical_width: Optional[int] = None
    scale_factor: Optional[int] = None


class DeviceHardwareInfo(BaseModel):
    """
    The Device Hardware Information object contains details and specifications of the physical components that make up a device. This information provides an overview of the hardware capabilities, configuration, and characteristics of the device.

    Attributes:
    - BIOS Date (bios_date) [Optional]: The BIOS date.
    - BIOS Manufacturer (bios_manufacturer) [Optional]: The BIOS manufacturer.
    - BIOS Version (bios_ver) [Optional]: The BIOS version.
    - CPU Bits (cpu_bits) [Optional]: The number of bits used for addressing in memory.
    - CPU Cores (cpu_cores) [Optional]: The number of processor cores in all installed processors.
    - CPU Count (cpu_count) [Optional]: The number of physical processors on a system.
    - Chassis (chassis) [Optional]: The system enclosure or physical form factor.
    - Desktop Display (desktop_display) [Optional]: The desktop display affiliated with the event.
    - Keyboard Information (keyboard_info) [Optional]: Detailed information about the keyboard.
    - Processor Speed (cpu_speed) [Optional]: The speed of the processor in Mhz.
    - Processor Type (cpu_type) [Optional]: The processor type.
    - RAM Size (ram_size) [Optional]: The total amount of installed RAM, in Megabytes.
    - Serial Number (serial_number) [Optional]: The device manufacturer serial number.
    """

    bios_date: Optional[str] = None
    bios_manufacturer: Optional[str] = None
    bios_ver: Optional[str] = None
    cpu_bits: Optional[int] = None
    cpu_cores: Optional[int] = None
    cpu_count: Optional[int] = None
    chassis: Optional[str] = None
    desktop_display: Optional[str] = None
    keyboard_info: Optional[str] = None
    cpu_speed: Optional[int] = None
    cpu_type: Optional[str] = None
    ram_size: Optional[int] = None
    serial_number: Optional[str] = None



from enum import IntEnum


class SeverityID(IntEnum):
    """
    The normalized identifier of the event/finding severity.

    The normalized severity is a measurement the effort and expense required to manage and resolve an event or incident. Smaller numerical values represent lower impact events, and larger numerical values represent higher impact events.

    0 Unknown: The event/finding severity is unknown.
    1 Informational: Informational message. No action required.
    2 Low: The user decides if action is needed.
    3 Medium: Action is required but the situation is not serious at this time.
    4 High: Action is required immediately.
    5 Critical: Action is required immediately and the scope is broad.
    6 Fatal: An error occurred but it is too late to take remedial action.
    99 Other: The event/finding severity is not mapped. See the severity attribute, which contains a data source specific value.
    """

    Unknown = 0
    Informational = 1
    Low = 2
    Medium = 3
    High = 4
    Critical = 5
    Fatal = 6
    Other = 99



from enum import IntEnum
from typing import Optional

from pydantic import BaseModel


class TypeID(IntEnum):
    Unknown = 0
    LDAP_Account = 1
    Windows_Account = 2
    AWS_IAM_User = 3
    AWS_IAM_Role = 4
    GCP_Account = 5
    Azure_AD_Account = 6
    MacOS_Account = 7
    Apple_Account = 8
    Linux_Account = 9
    AWS_Account = 10
    Other = 99


class Account(BaseModel):
    """
    The Account object contains details about the account that initiated or performed a specific activity within a system or application.

    Attributes:
    - Name (name) [Recommended]: The name of the account (e.g. GCP Account Name).
    - Type (type) [Optional]: The account type, normalized to the caption of 'account_type_id'. In the case of 'Other', it is defined by the event source.
    - Type ID (type_id) [Recommended]: The normalized account type identifier.
    - Unique ID (uid) [Recommended]: The unique identifier of the account (e.g. AWS Account ID).
    - Labels (labels) [Optional]: The labels associated with the account.

    """

    name: str
    type: Optional[str] = None
    type_id: TypeID
    uid: str
    labels: Optional[list[str]] = None



from enum import IntEnum


class VerdictID(IntEnum):
    """
    The normalized verdict (or status) ID of the evidence associated with the security detection. For example, Microsoft Graph Security Alerts contain a verdict enumeration for each type of evidence associated with the Alert. This is typically set by an automated investigation process or an analyst/investigator assigned to the finding.
        0	Unknown: The type is unknown.
        1	False Positive: The verdict for the evidence has been identified as a False Positive.
        2	True Positive: The verdict for the evidence has been identified as a True Positive.
        3	Disregard: The verdict for the evidence is that is should be Disregarded.
        4	Suspicious: The verdict for the evidence is that the behavior has been identified as Suspicious.
        5	Benign: The verdict for the evidence is that the behavior has been identified as Benign.
        6	Test: The evidence is part of a Test, or other sanctioned behavior(s).
        7	Insufficient Data: There is insufficient data to render a verdict on the evidence.
        8	Security Risk: The verdict for the evidence is that the behavior has been identified as a Security Risk.
        9	Managed Externally: The verdict for the evidence is Managed Externally, such as in a case management tool.
        10	Duplicate: This evidence duplicates existing evidence related to this finding.
        99	Other: The type is not mapped. See the type attribute, which contains a data source specific value.
    """

    Unknown = 0
    FalsePositive = 1
    TruePositive = 2
    Disregard = 3
    Suspicious = 4
    Benign = 5
    Test = 6
    InsufficientData = 7
    SecurityRisk = 8
    ManagedExternally = 9
    Duplicate = 10
    Other = 99



from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.objects.container import Container


class ResponseElements(BaseModel):
    """
    Represents the elements of an API response, particularly useful in containerized applications for capturing the output of containers, the response data, any associated error codes or messages, communication flags, and a descriptive message about the response.

    Attributes:
    - Containers (containers) [Optional]: When working with containerized applications, the set of containers which write to the standard the output of a particular logging driver. For example, this may be the set of containers involved in handling api requests and responses for a containerized application.
    - Data(data) [Optional]: The additional data that is associated with the api response.
    - Error Code (error) [Optional]: Error code associated with the response.
    - Error Message	(error_message) [Optional]: Descriptive error message.
    - Flags	(flags) [Optional]: The list of communication flags, normalized to the captions of the flag_ids values. In the case of 'Other', they are defined by the event source.
    - Message (message) [Optional]: The description of the event/finding, as defined by the source.
    - Response Code	(code) [Optional]: The numeric response sent to a request.

    """

    containers: Optional[list[Container]] = None
    data: Optional[dict[str, object]] = None
    error: Optional[str] = None
    error_message: Optional[str] = None
    flags: Optional[list[str]] = None
    message: Optional[str] = None
    code: Optional[int] = None



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from py_ocsf_models.events.findings.category_uid import CategoryUID
from py_ocsf_models.events.findings.class_uid import ClassUID
from py_ocsf_models.events.findings.detection_finding_type_id import (
    DetectionFindingTypeID,
)
from py_ocsf_models.events.findings.finding import Finding
from py_ocsf_models.events.findings.impact_id import ImpactID
from py_ocsf_models.events.findings.risk_level_id import RiskLevelID
from py_ocsf_models.events.findings.status_id import StatusID
from py_ocsf_models.objects.api import API
from py_ocsf_models.objects.cloud import Cloud
from py_ocsf_models.objects.evidence_artifacts import EvidenceArtifacts
from py_ocsf_models.objects.remediation import Remediation
from py_ocsf_models.objects.resource_details import ResourceDetails
from py_ocsf_models.objects.vulnerability_details import VulnerabilityDetails


class DetectionFinding(Finding, BaseModel):
    """
    A Detection Finding describes detections or alerts generated by security products using correlation engines, detection engines or other methodologies. Note: if the product is a security control, the security_control profile should be applied and its attacks information should be duplicated into the finding_info object.

    Attributes:
    - Affected Resources (resources) [Recommended]: Describes details about resources that were the target of the activity that triggered the finding.
    - Category (category_name): The event category name, as defined by category_uid value: Findings.
    - Class (class_name) [Optional]: The event class name, as defined by class_uid value: Detection Finding.
    - Class ID (class_uid): The unique identifier of a class. A Class describes the attributes available in an event.
    - Cloud (cloud) [Optional]: Describes details about the Cloud environment where the event was originally created or logged.
    - Count (count) [Optional]: Number of times similar events occurred within a specified timeframe.
    - Duration (duration) [Optional]: Time span of the event, from start to end, in milliseconds.
    - Event Time (time) [Required]: The standardized time when the event occurred or the finding was created.
    - Event Time (time_dt) [Optional]: The standardized time when the event occurred or the finding was created, in datetime format.
    - Evidence Artifacts (evidences) [Optional]: Artifacts related to the security detection activities.
    - Impact (impact) [Optional]: The impact, normalized to the caption of the impact_id value. In the case of 'Other', it is defined by the event source.
    - Impact Score (impact_score) [Optional]: The impact of the finding, valid range 0-100.
    - Impact ID (impact_id) [Optional]: The normalized impact of the finding.
    - Remediation Guidance (remediation) [Optional]: Suggested steps to address the finding.
    - Risk Level (risk_level) [Optional]: The risk level, normalized to the caption of the risk_level_id value. In the case of 'Other', it is defined by the event source.
    - Risk Level ID (risk_level_id) [Optional]: The normalized risk level id.
    - Risk Score (risk_score) [Optional]: The risk score as reported by the event source.
    - Risk Details (risk_details) [Optional]: Additional details about the risk.
    - Status ID (status_id) [Optional]: The normalized identifier of the event/finding severity.
    - Timezone Offset (timezone_offset) [Optional]: Difference in minutes from UTC.
    - Type ID (type_uid): The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: class_uid * 100 + activity_id.
    - Type Name (type_name) [Optional]: The event/finding type name, as defined by the type_uid.
    - Vulnerabilities (vulnerabilities) [Optional]: Vulnerabilities identified in the finding.

    If the Cloud profile is needed:
        - API Details (api) [Optional]: Describes details about a typical API (Application Programming Interface) call.
        - Cloud (cloud): Describes details about the Cloud environment where the event was originally created or logged
    """

    resources: Optional[list[ResourceDetails]] = None
    category_name: str = CategoryUID.Findings.name
    category_uid: int = CategoryUID.Findings.value
    class_name: Optional[str] = "Detection Finding"
    class_uid: int = ClassUID.DetectionFinding
    cloud: Optional[Cloud] = None
    api: Optional[API] = None
    count: Optional[int] = None
    duration: Optional[int] = None
    evidences: Optional[list[EvidenceArtifacts]] = None
    impact: Optional[str] = None
    impact_score: Optional[int] = None
    impact_id: Optional[ImpactID] = None
    remediation: Optional[Remediation] = None
    risk_level: Optional[str] = None
    risk_level_id: Optional[RiskLevelID] = None
    risk_score: Optional[int] = None
    risk_details: Optional[str] = None
    status_id: Optional[StatusID] = None
    time: int
    time_dt: Optional[datetime] = None
    timezone_offset: Optional[int] = None
    type_uid: DetectionFindingTypeID
    type_name: Optional[str] = None
    vulnerabilities: Optional[list[VulnerabilityDetails]] = None
